/*
 * Copyright 2009 Dukascopy® (Suisse) SA. All rights reserved.
 * DUKASCOPY PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * 
 *  This Dukascopy based code(HeikinAshi indicator) was modified by Schweizerkatz, no warranty. 
 */
package com.dukascopy.indicators;

import com.dukascopy.api.indicators.IDrawingIndicator;
import com.dukascopy.api.indicators.IIndicator;
import com.dukascopy.api.indicators.IIndicatorContext;
import com.dukascopy.api.indicators.IIndicatorDrawingSupport;
import com.dukascopy.api.indicators.IndicatorInfo;
import com.dukascopy.api.indicators.IndicatorResult;
import com.dukascopy.api.indicators.InputParameterInfo;
import com.dukascopy.api.indicators.OptInputParameterInfo;
import com.dukascopy.api.indicators.OutputParameterInfo;
import com.dukascopy.api.indicators.BooleanOptInputDescription;
import com.dukascopy.api.indicators.IntegerRangeDescription;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Shape;
import java.awt.Stroke;
import java.util.Map;

/**
 * <b>NOTE: </b> The calculate logic of this indicator is implemented in JavaScript.
 * Please, update the corresponding JS code in case of updating of this class.
 * Created by: S.Vishnyakov
 * Date: Jul 23, 2009
 * Time: 11:48:57 AM
 */
public class CBar implements IIndicator, IDrawingIndicator {
    public static final int OPEN = 0;
    public static final int CLOSE = 1;
    public static final int HIGH = 2;
    public static final int LOW = 3;

    private IndicatorInfo indicatorInfo;
    private InputParameterInfo[] inputParameterInfos;
    private OptInputParameterInfo[] optInputParameterInfos;
    private OutputParameterInfo[] outputParameterInfos;

    private double[][][] inputs = new double[1][][];
    private Object[][] outputs = new Object[2][];
    private int WRange = 0;

//------------------------------------------------------------------------------------------------------------------------------------------------------

    public void onStart(IIndicatorContext context) {
  
        indicatorInfo = new IndicatorInfo("CBar", "CBar", "Custom indicators", true, false, true, 1, 1, 2);
        inputParameterInfos = new InputParameterInfo[] { new InputParameterInfo("Price", InputParameterInfo.Type.PRICE) };
        optInputParameterInfos = new OptInputParameterInfo[] { new OptInputParameterInfo("Width", OptInputParameterInfo.Type.OTHER, new IntegerRangeDescription(0, 0, 2, 1)) };
        outputParameterInfos = new OutputParameterInfo[] { 
                new OutputParameterInfo("CBar", OutputParameterInfo.Type.OBJECT, OutputParameterInfo.DrawingStyle.LINE) {{ setDrawnByIndicator(true); }}, 
                new OutputParameterInfo("HLPeak", OutputParameterInfo.Type.OBJECT, OutputParameterInfo.DrawingStyle.LINE) {{ setDrawnByIndicator(true); }} };
    }

//-------------------------------------------------------------------------------------------------------------------------------------------------------

    public IndicatorResult calculate(int startIndex, int endIndex) {
 
        if (startIndex - getLookback() < 0) { startIndex -= startIndex - getLookback(); }
        if (startIndex > endIndex) { return new IndicatorResult(0, 0); }

        double[][] prices = inputs[0];

        int resIndex = 0;
        for (int i = startIndex; i <= endIndex; i++, resIndex++) {

            double[] newHACandle = new double[4];
            
            newHACandle[OPEN] = prices[OPEN][i];
            newHACandle[CLOSE] = prices[CLOSE][i];
            newHACandle[HIGH] = prices[HIGH][i];
            newHACandle[LOW] = prices[LOW][i];
            
            outputs[0][resIndex] = newHACandle;
            outputs[1][resIndex] = newHACandle;
        }
        
        return new IndicatorResult(startIndex, resIndex);
    }

//--------------------------------------------------------------------------------------------------------------------------------------------------------

    public Point drawOutput(
            Graphics g, 
            int outputIdx, 
            Object values2, 
            Color color, 
            Stroke stroke, 
            IIndicatorDrawingSupport indicatorDrawingSupport, 
            java.util.List<Shape> shapes, 
            Map<Color, java.util.List<Point>> handles) 
    {
        
        if (values2 != null) 
        {
            Object[] values = (Object[]) values2;
      
            // check for visible candles
            for (int i = indicatorDrawingSupport.getIndexOfFirstCandleOnScreen(), j = 0; j < indicatorDrawingSupport.getNumberOfCandlesOnScreen(); i++, j++) 
            {
                if (values[i] != null) 
                {
                    double[] candle = (double[]) values[i];
                    int candleMiddle = (int)indicatorDrawingSupport.getMiddleOfCandle(i);
                    int halfCandle = (int)indicatorDrawingSupport.getCandleWidthInPixels() / 2;
                    
                    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

                    if(outputIdx == 0 && outputParameterInfos[outputIdx].isShowOutput()) 
                    {
                        g.setColor(outputParameterInfos[outputIdx].getColor());
                        g.fillRect(candleMiddle - halfCandle - WRange, (int)indicatorDrawingSupport.getYForValue(candle[HIGH]), (int)indicatorDrawingSupport.getCandleWidthInPixels() + WRange, 
                                  (int)Math.abs(indicatorDrawingSupport.getYForValue(candle[HIGH]) - indicatorDrawingSupport.getYForValue(candle[LOW])) + 1);
    
                        double higher = (candle[OPEN] > candle[CLOSE]) ? candle[OPEN] : candle[CLOSE]; 
                        double lower = (candle[OPEN] > candle[CLOSE]) ? candle[CLOSE] : candle[OPEN];

                        g.setColor(outputParameterInfos[outputIdx].getColor2());
                        g.fillRect(candleMiddle - halfCandle - WRange, (int)indicatorDrawingSupport.getYForValue(higher), (int)indicatorDrawingSupport.getCandleWidthInPixels() + WRange, 
                                (int)Math.abs(indicatorDrawingSupport.getYForValue(higher) - indicatorDrawingSupport.getYForValue(lower)) + 1);
                    }

                    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

                    if(outputIdx == 1 && outputParameterInfos[outputIdx].isShowOutput()) 
                    {
                        g.setColor(outputParameterInfos[outputIdx].getColor());
    
                        g.drawLine(candleMiddle - halfCandle - WRange, (int)indicatorDrawingSupport.getYForValue(candle[HIGH]), 
                                candleMiddle + halfCandle + WRange, (int)indicatorDrawingSupport.getYForValue(candle[HIGH]));
        
                        g.setColor(outputParameterInfos[outputIdx].getColor2());

                        g.drawLine(candleMiddle - halfCandle - WRange, (int)indicatorDrawingSupport.getYForValue(candle[LOW]), 
                                candleMiddle + halfCandle + WRange, (int)indicatorDrawingSupport.getYForValue(candle[LOW]));
                    }
                }
            }
        }
        
        return null;
    }

//----------------------------------------------------------------------------------------------------------------------------------------------------------------
    
    public IndicatorInfo getIndicatorInfo() { return indicatorInfo; }

    public InputParameterInfo getInputParameterInfo(int index) {
        if (index <= inputParameterInfos.length) {
            return inputParameterInfos[index];
        }
        return null;
    }
    public OptInputParameterInfo getOptInputParameterInfo(int index) { 
        if (index <= optInputParameterInfos.length) {
            return optInputParameterInfos[index];
        }
        return null;
    }
    public OutputParameterInfo getOutputParameterInfo(int index) {
        if (index <= outputParameterInfos.length) {
            return outputParameterInfos[index];
        }
        return null;
    }
 
    public void setInputParameter(int index, Object array) { inputs[0] = (double[][]) array; }
    public void setOptInputParameter(int index, Object value) { WRange = Integer.valueOf(String.valueOf(value)).intValue(); }
    public void setOutputParameter(int index, Object array) { outputs[index] = (Object[]) array; }

    public int getLookback() { return 1; }
    public int getLookforward() { return 0; }
}
